@@ -21,8 +21,14 @@
  *    Jean-Francois Wauthy (option panel for choice between icon/text)
  *    Jasper Huijsmans (menu placement function, toggle button, scaled image
  *                      fixes)
+ *    Olivier Fourdan  (remote popup)
  */
 
+#include <gtk/gtk.h>
+#include <glib.h>
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -48,11 +54,17 @@
 #include <libxfce4panel/xfce-panel-convenience.h>
 
 #include "desktop-menu-stub.h"
+#include "xfdesktop-common.h"
+#include "xfce4-popup-menu.h"
 
 #define BORDER 8
 #define DEFAULT_BUTTON_ICON  DATADIR "/pixmaps/xfce4_xicon1.png"
 #define DEFAULT_BUTTON_TITLE "Xfce Menu"
 
+#ifndef CHECK_RUNNING_PLUGIN
+#define CHECK_RUNNING_PLUGIN 0
+#endif
+
 typedef struct _DMPlugin {
     XfcePanelPlugin *plugin;
     
@@ -290,8 +302,19 @@ dmp_position_menu (GtkMenu *menu, int *x, int *y, gboolean *push_in,
             break;
         
         default:  /* floating */
+        {
+            GdkScreen *screen = NULL;
+            gint screen_width, screen_height;
+
             gdk_display_get_pointer(gtk_widget_get_display(GTK_WIDGET(dmp->plugin)),
-                                                           NULL, x, y, NULL);
+                                                           &screen, x, y, NULL);
+            screen_width = gdk_screen_get_width(screen);
+            screen_height = gdk_screen_get_height(screen);
+            if ((*x + req.width) > screen_width)
+                *x -= req.width;
+            if ((*y + req.height) > screen_height)
+                *y -= req.height;
+        }
     }
 
     if (*x < 0)
@@ -317,26 +340,16 @@ menu_deactivated(GtkWidget *menu, gpointer user_data)
     g_signal_handler_disconnect(menu, id);
 }
 
-static gboolean
-dmp_popup(GtkWidget *w,
-          GdkEventButton *evt,
-          gpointer user_data)
+static void
+menu_activate(DMPlugin *dmp)
 {
     GtkWidget *menu;
-    DMPlugin *dmp = user_data;
+    GtkWidget *button;
     
-    if(evt->button != 1 || ((evt->state & GDK_CONTROL_MASK)
-                            && !(evt->state & (GDK_MOD1_MASK|GDK_SHIFT_MASK
-                                               |GDK_MOD4_MASK))))
-    {
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);
-        return FALSE;
-    }
-
+    button = dmp->button;
     if(!dmp->desktop_menu) {
         g_critical("dmp->desktop_menu is NULL - module load failed?");
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);
-        return TRUE;
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
     }
     
     if(xfce_desktop_menu_need_update(dmp->desktop_menu))
@@ -349,12 +362,60 @@ dmp_popup(GtkWidget *w,
         id = g_signal_connect(menu, "deactivate", 
                 G_CALLBACK(menu_deactivated), dmp);
         g_object_set_data(G_OBJECT(menu), "sig_id", GUINT_TO_POINTER(id));
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+        xfce_panel_plugin_register_menu (dmp->plugin, GTK_MENU(menu));
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
                        (GtkMenuPositionFunc)dmp_position_menu, dmp,
                        1, gtk_get_current_event_time());
     } else
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
+}
+
+static gboolean
+dmp_message_received(GtkWidget *w, 
+                        GdkEventClient *evt, 
+                        gpointer user_data)
+{
+    DMPlugin *dmp = user_data;
+    GdkScreen *gscreen;
+    GdkWindow *root;
+    GtkWidget *button;
+
+    button = dmp->button;
+    gscreen = gtk_widget_get_screen (button);
+    root = gdk_screen_get_root_window(gscreen);
+    if(!xfdesktop_popup_grab_available(root, GDK_CURRENT_TIME))
+    {
+        g_critical("Unable to get keyboard/mouse grab.");
+        return FALSE;
+    }
+
+    if(evt->data_format == 8) {
+        if(strcmp(XFCE_MENU_MESSAGE, evt->data.b) == 0) {
+            menu_activate(dmp);
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+static gboolean
+dmp_popup(GtkWidget *w,
+          GdkEventButton *evt,
+          gpointer user_data)
+{
+    DMPlugin *dmp = user_data;
+    
+    if(evt->button != 1 || ((evt->state & GDK_CONTROL_MASK)
+                            && !(evt->state & (GDK_MOD1_MASK|GDK_SHIFT_MASK
+                                               |GDK_MOD4_MASK))))
+    {
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);
+        return FALSE;
+    }
+
+    menu_activate(dmp);
 
     return TRUE;
 }
@@ -860,10 +921,67 @@ dmp_create_options(XfcePanelPlugin *plugin, DMPlugin *dmp)
     gtk_widget_show(dlg);
 }
 
+#if CHECK_RUNNING_PLUGIN
+static gboolean 
+desktop_menu_plugin_check (GdkScreen *gscreen)
+{
+    gchar selection_name[32];
+    Atom selection_atom;
+    
+    g_snprintf(selection_name, 
+               sizeof(selection_name), 
+               XFCE_MENU_SELECTION"%d", 
+               gdk_screen_get_number (gscreen));
+    selection_atom = XInternAtom(GDK_DISPLAY(), selection_name, False);
+
+    if(XGetSelectionOwner(GDK_DISPLAY(), selection_atom)) {
+        xfce_info (_("There is already a panel menu registered for this screen"));
+        return FALSE;
+    }
+
+    return TRUE;
+}
+#endif
+
+static gboolean
+dmp_set_selection(DMPlugin *dmp)
+{
+    GdkScreen *gscreen;
+    gchar selection_name[32];
+    Atom selection_atom;
+    GtkWidget *win;
+    Window xwin;
+
+    win = gtk_invisible_new();
+    gtk_widget_realize(win);
+    xwin = GDK_WINDOW_XID(GTK_WIDGET(win)->window);
+
+    gscreen = gtk_widget_get_screen (win);
+    g_snprintf(selection_name, 
+               sizeof(selection_name), 
+               XFCE_MENU_SELECTION"%d", 
+               gdk_screen_get_number (gscreen));
+    selection_atom = XInternAtom(GDK_DISPLAY(), selection_name, False);
+
+    if(XGetSelectionOwner(GDK_DISPLAY(), selection_atom)) {
+        gtk_widget_destroy (win);
+        return FALSE;
+    }
+
+    XSelectInput(GDK_DISPLAY(), xwin, PropertyChangeMask);
+    XSetSelectionOwner(GDK_DISPLAY(), selection_atom, xwin, GDK_CURRENT_TIME);
+
+    g_signal_connect(G_OBJECT(win), "client-event",
+                     G_CALLBACK(dmp_message_received), dmp);
+    
+    return TRUE;
+}
+
 static DMPlugin *
 dmp_new(XfcePanelPlugin *plugin)
 {
     DMPlugin *dmp = g_new0(DMPlugin, 1);
+
     dmp->plugin = plugin;
     dmp_read_config(plugin, dmp);
     
@@ -903,10 +1021,8 @@ dmp_new(XfcePanelPlugin *plugin)
     }
     g_signal_connect(G_OBJECT(dmp->button), "button-press-event",
             G_CALLBACK(dmp_popup), dmp);
-    
-    xfce_panel_plugin_add_action_widget(plugin, dmp->button);
-    gtk_container_add(GTK_CONTAINER(plugin), dmp->button);
-    
+    dmp_set_selection(dmp);
+
     return dmp;
 }
 
@@ -914,12 +1030,26 @@ static void
 desktop_menu_plugin_construct(XfcePanelPlugin *plugin)
 {
     DMPlugin *dmp;
+    GtkWidget *mi, *img;
     
     xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
     
     if(!(dmp = dmp_new(plugin)))
         exit(1);
     
+    xfce_panel_plugin_add_action_widget(plugin, dmp->button);
+    gtk_container_add(GTK_CONTAINER(plugin), dmp->button);
+    
+    /* Add edit menu option to right click menu */
+    img = gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
+    gtk_widget_show(img);
+    mi = gtk_image_menu_item_new_with_label(_("Edit Menu"));
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    gtk_widget_show(mi);
+    xfce_panel_plugin_menu_insert_item(plugin, GTK_MENU_ITEM(mi));
+    g_signal_connect(G_OBJECT(mi), "activate", 
+                     G_CALLBACK(dmp_edit_menu_clicked_cb), dmp);
+    
     g_signal_connect(plugin, "free-data",
                      G_CALLBACK(dmp_free), dmp);
     g_signal_connect(plugin, "save",
@@ -936,4 +1066,9 @@ desktop_menu_plugin_construct(XfcePanelPlugin *plugin)
     dmp_set_size(plugin, xfce_panel_plugin_get_size(plugin), dmp);
 }
 
+#if CHECK_RUNNING_PLUGIN
+XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL_WITH_CHECK (desktop_menu_plugin_construct,
+                                                desktop_menu_plugin_check)
+#else
 XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL(desktop_menu_plugin_construct)
+#endif
